# Generated by Django 6.0.2 on 2026-02-26 20:15

import django.db.models.deletion
import django_prometheus.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("tests", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="TaskResult",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("task_id", models.CharField(max_length=255, unique=True)),
                ("result", models.IntegerField()),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                (
                    "order",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="results",
                        to="tests.order",
                    ),
                ),
            ],
            options={
                "ordering": ["-created_at"],
            },
            bases=(
                django_prometheus.models.ExportModelOperationsMixin("taskresult"),
                models.Model,
            ),
        ),
    ]
